﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace TaskManager
{
    /// <summary>
    /// 任务调度器
    /// </summary>
    public class BatchTaskScheduler
    {
        public BatchTaskScheduler()
        {
            CurrentProcessTasks=new ObservableCollection<TaskBase>();
        }
        /// <summary>
        /// 当前所有执行的任务
        /// </summary>
        public IList<TaskBase> CurrentProcessTasks { get; set; }
        /// <summary>
        /// 添加一个临时任务
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="taskName">任务名称</param>
        /// <param name="enumable">任务枚举器</param>
        /// <param name="action">对枚举的每一个元素执行的操作</param>
        /// <param name="contineAction">枚举完成后执行的操作</param>
        /// <param name="count">可选，枚举总数量，用于指示进度条</param>
        /// <param name="autoStart">是否自动运行</param>
        public void AddTempTask<T>(string taskName, IEnumerable<T> enumable, Action<T> action, Action<int> contineAction = null, int count = -1, bool autoStart = true)
        {
            var tempTask = new TemporaryTask();
            tempTask.Scheduler = this;
            tempTask.Name = taskName;
            tempTask.TaskAction = () =>
            {

                if (enumable is ICollection<T>)
                {
                    count = (enumable as ICollection<T>).Count;  //此处可能能够获取整个枚举的大小
                }
                if (count == 0)
                    count = -1;
                var finish = false;
                foreach (var r in enumable)
                {
                    if (action != null)
                        action(r);
                    tempTask.CheckWait();  
                    if (r is int)
                    {
                        tempTask.CurrentIndex = Convert.ToInt32(r);
                    }
                    else
                    {
                        tempTask.CurrentIndex++;
                    }

                    if (count != -1)
                    {
                        tempTask.Percent = tempTask.CurrentIndex * 100 / count;  //计算进度条位置
                    }
                    if (tempTask.CheckCancel())
                    {
                        finish = true;
                        break;
                    }

                }
                if (finish == true)
                    tempTask.Percent = 100;
                if (contineAction != null)
                {
                    ControlExtended.UIInvoke(() => contineAction(tempTask.CurrentIndex));
                }

            };
            this.CurrentProcessTasks.Add(tempTask);
            if (autoStart == true)
            {

                tempTask.Start();
            }


        }
    }
}
