﻿using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows;

namespace TaskManager
{
    /// <summary>
    ///     MainWindow.xaml 的交互逻辑
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            Scheduler = new BatchTaskScheduler();
            DataContext = Scheduler;
        }

        public BatchTaskScheduler Scheduler { get; set; }


        public IEnumerable<int> TestTask(int count)
        {
            for (int j = 0; j < count; j++)
            {
                Thread.Sleep(300);
                yield return j;
            }
        }

        public IEnumerable<int> WriteFileTask(string filename, int count)
        {
            var fs = new FileStream(filename, FileMode.OpenOrCreate);

            var sw = new StreamWriter(new BufferedStream(fs), Encoding.Default);

            int j = 0;
            using (var dis = new DisposeHelper(() =>
            {
                if (j != count)
                    MessageBox.Show("用户取消操作，点确定开始关闭文件流");
                sw.Close();

                fs.Close();
            }))
            {
                for (j = 0; j < count; j++)
                {
                    Thread.Sleep(100);
                    sw.WriteLine("这个数据是" + j);
                    yield return j;
                }
            }
        }

        public IEnumerable<int> WriteFileUnsafe(string filename, int count)
        {
            var fs = new FileStream(filename, FileMode.OpenOrCreate);

            var sw = new StreamWriter(new BufferedStream(fs), Encoding.Default);

            int j = 0;


            for (j = 0; j < count; j++)
            {
                Thread.Sleep(100);
                sw.WriteLine("这个数据是" + j);
                yield return j;
            }
            sw.Close();

            fs.Close();
        }


        private void Button_Click_1(object sender, RoutedEventArgs e)
        {
            int total = 15;
            Scheduler.AddTempTask("任务1：延时测试", TestTask(total), null,
                result => MessageBox.Show(string.Format("延时测试任务已经完成,迭代位置{0}", result)), total);
        }

        private void Button_Click_2(object sender, RoutedEventArgs e)
        {
            TaskBase t = Scheduler.CurrentProcessTasks.FirstOrDefault(d => d.Name.Contains("任务2"));
            if (t != null)
            {
                MessageBox.Show("任务2只能同时进行一个");
                return;
            }
            string fileName = "Test.txt";
            Scheduler.AddTempTask("任务2：写入文件", WriteFileTask(fileName, 100), null,
                d =>
                {
                    if (d == 100)
                    {
                        MessageBox.Show(string.Format("写入文件测试任务已经完成,迭代位置{0}", d));
                    }

                    Process.Start(fileName);
                }, 100);
        }

        private void Button_Click_3(object sender, RoutedEventArgs e)
        {
            TaskBase t = Scheduler.CurrentProcessTasks.FirstOrDefault(d => d.Name.Contains("任务2"));
            if (t != null)
            {
                t.Cancel();
            }

            else
            {
                MessageBox.Show("当前任务队列中没有添加任务2");
            }
        }
    }
}