﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace TaskManager
{
    public abstract class TaskBase : PropertyChangeNotifier
    {
        private bool _isPause;
        private bool _isStart;
        private AutoResetEvent autoReset;
        private int currentIndex;
        private string name;
        private int _percent;

        protected TaskBase()
        {
            autoReset = new AutoResetEvent(false);
            

        }

        /// <summary>
        /// 当任务结束后，自动取消任务
        /// </summary>
        [Browsable(false)]
        public bool AutoDelete { get; set; }

        public bool CheckWait()
        {
            if (IsPause)
            {
                autoReset.WaitOne();
                return true;
            }
            return false;
        }
        [Category("遍历状态")]
        [DisplayName("当前位置")]
        public int CurrentIndex
        {
            get { return currentIndex; }
            set
            {
                if (currentIndex != value)
                {
                    currentIndex = value;
                    OnPropertyChanged("CurrentIndex");
                }
            }
        }

       

        /// <summary>
        ///     该计算任务的介绍
        /// </summary>
        [DisplayName("任务描述")]
     
        public string Description { get; set; }

        [DisplayName("分组")]
      
        public string Group { get; set; }

        [DisplayName("名称")]
    
        public string Name
        {
            get { return name; }
            set
            {
                if (name != value)
                {
                    if (IsStart)
                        return;
                    name = value;
                    OnPropertyChanged("Name");
                }
            }
        }

        [Browsable(false)]
        public bool IsStart
        {
            get { return _isStart; }
            set
            {
                if (_isStart != value)
                {
                    _isStart = value;
                    OnPropertyChanged("IsStart");
                }
            }
        }

        public virtual void Start()
        {
            CancellationToken = new CancellationTokenSource();
            CancellationToken.Token.Register(
                () =>
                {
                   
                    if (AutoDelete == true)
                        Remove();
                });
            IsStart = true;

        }

        public virtual void Remove()
        {
            ControlExtended.UIInvoke(() => Scheduler.CurrentProcessTasks.Remove(this));
            if (CancellationToken != null)
                CancellationToken.Cancel();
            autoReset.Close();
            

        }
      
        public BatchTaskScheduler Scheduler { get; set; }

        public void Cancel()
        {
            IsStart = false;

            try
            {
                CancellationToken.Cancel();
            }
            catch (Exception ex)
            {
                
              
            }
         

        }

        public bool CheckCancel()
        {
            return CancellationToken.IsCancellationRequested;
        }
       
        public bool IsPause
        {
            get { return _isPause; }
            set
            {
                if (_isPause != value)
                {
                    _isPause = value;
                    if (value)
                    {
                        autoReset.Reset();
                    }
                    else
                    {
                        autoReset.Set();
                    }

                    OnPropertyChanged("IsPause");
                }
            }
        }

        /// <summary>
        ///     完成度
        /// </summary>
        [Browsable(false)]
        public int Percent
        {
            get { return _percent; }
            set
            {
                _percent = value;
                OnPropertyChanged("Percent");
            }
        }

        /// <summary>
        ///     是否需要取消任务
        /// </summary>
        protected CancellationTokenSource CancellationToken { get; set; }

       
    }
}
